// ==============================================================
//	This file is part of Glest Shared Library (www.glest.org)
//
//	Copyright (C) 2005 Matthias Braun <matze@braunis.de>
//
//	You can redistribute this code and/or modify it under 
//	the terms of the GNU General Public License as published 
//	by the Free Software Foundation; either version 2 of the 
//	License, or (at your option) any later version
// ==============================================================

#ifndef _SHARED_PLATFORM_SOCKET_H_
#define _SHARED_PLATFORM_SOCKET_H_

#include <string>

#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>

using std::string;

namespace Shared{ namespace Platform{

// =====================================================
//	class Socket
// =====================================================

class Socket {
protected:
	int sock;
	
public:
	Socket(int sock);
	Socket();
	~Socket();

	void close();
	void send(const void *data, int dataSize);
	void receive(void *data, int dataSize);
	
	//void setBlock(bool block);
	//bool isReadable();

	string getHostname() const;

protected:
	static void throwException(const string &str);
};

// =====================================================
//	class ClientSocket
// =====================================================

class ClientSocket: public Socket{
public:
	void connect(const string &host, int port);
};

// =====================================================
//	class ServerSocket
// =====================================================

class ServerSocket: public Socket{
public:
	void bind(int port);
	void listen();
	Socket *accept();
};

}}//end namespace

#endif
